// This file is part of LeebA, a lightweight C++ template library
// for convex optimization and control.
//
// Copyright (C) 2008 Timothee Hunter <tjhunter@stanford.edu>
//
// LeebA is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public
// License as published by the Free Software Foundation; either
// version 3 of the License, or (at your option) any later version.
//
// LeebA is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License  for
// more details.
//
// You should have received a copy of the GNU Lesser General Public
// License along with LeebA in the file COPYING.LESSER at the root of the
// LeebA package. If not, see <http://www.gnu.org/licenses/>.
/*
 * Symbol.h
 *
 *  Created on: Dec 19, 2008
 *      Author: Timothy Hunter <tjhunter@stanford.edu>
 */

#include <string>
#include <boost/shared_ptr.hpp>
#include <boost/functional/hash.hpp>
#include <iostream>

#ifndef SYMBOL_H_
#define SYMBOL_H_

namespace Eigen
{

/**
  * @brief A small class to represent a symbol.
  *
  * @note Depends on boost::shared_ptr
  */
class Symbol
{
public:
  Symbol(const std::string & n="") : name_(new std::string(n)) {}

  const std::string & name() const { return *name_; }

  friend bool operator==(const Symbol & s1, const Symbol & s2)
  {
    return s1.name_==s2.name_;
  }

  friend std::size_t hash_value(const Symbol & s)
  {
    boost::hash<int> hasher;
    return hasher(long(s.name_.get()));
  }

private:
  boost::shared_ptr<std::string> name_;
};

inline std::ostream & operator<<(std::ostream & s, const Eigen::Symbol & sym)
{
  return s<<sym.name();
}

}


#endif /* SYMBOL_H_ */
